#include "arp.h"

#include <timerms.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <netinet/in.h>

/* Cabecera ARP */

struct arp_message {

  uint16_t hardware_type; /* Espacio de dirección de hardware */
  uint16_t protocol_type; /* Espacio de dirección del protocolo */
  uint8_t  hardware_size; /* Largo en bytes de cada dirección de hardware */
  uint8_t protocol_size;  /* Largo en bytes de cada dirección de protocolo */
  uint16_t opcode;        /* Opcode */

  mac_addr_t src_addr_mac;  /* Dirección MAC origen */
  ipv4_addr_t src_addr_ip; /* Dirección IP origen*/
  mac_addr_t dest_addr_mac; /* Dirección MAC destino*/
  ipv4_addr_t dest_addr_ip; /* Dirección IP destino*/

  /* NOTA: El campo "Frame Checksum" (FCS) no está incluido en la estructura
     porque lo añade automáticamente la tarjeta de red. */
};



int arp_resolve (eth_iface_t *iface, ipv4_addr_t my_ip_addr, ipv4_addr_t ip_addr, mac_addr_t mac_addr){
  arp_message_t arp_request;

  /* CAMBIAR NUMEROS POR VARIABLES*/

  arp_request.hardware_type = htons(ARP_MSG_HW_TYPE); /* Usamos htons() para que no se descoloquen los datos, en los campos que necesitan más de 1 byte*/
  arp_request.protocol_type = htons(ARP_MSG_PT_TYPE); /* 0x0800 quiere decir que usamos IP*/
  arp_request.hardware_size = MAC_ADDR_SIZE; /* Longitud de la MAC en bytes*/
  arp_request.protocol_size = IPv4_ADDR_SIZE; /* Longitud de la IP en bytes*/
  arp_request.opcode = htons(ARP_OPCODE_REQUEST); /* Request*/

  eth_getaddr(iface, arp_request.src_addr_mac);  /* Esta función permite obtener la dirección MAC de la interfaz Ethernet especificada*/
  memcpy(arp_request.src_addr_ip, my_ip_addr, IPv4_ADDR_SIZE);
  memset(arp_request.dest_addr_mac, 0x00, MAC_ADDR_SIZE);
  memcpy(arp_request.dest_addr_ip, ip_addr, IPv4_ADDR_SIZE);   /* Copia la IP destino que se pasa como parámetro en la estructura arp_message_t */

  int sent_message=0;
  sent_message= eth_send ( iface, MAC_BCAST_ADDR, ARP_ETH_TYPE, (unsigned char *)&arp_request, ARP_DATA_LENGTH);

  if (sent_message !=ARP_DATA_LENGTH)
  {  /*Si no se envían 28 bytes, debería dar error */
     printf("ERROR, no se puede enviar el mensaje\n");
     return -1;
  }

  int recv_message=0;
  unsigned char buffer[ARP_DATA_LENGTH];
  int buf_len=ARP_DATA_LENGTH;
  long int timeout=2000;
  arp_message_t * arp_reply_ptr = NULL;
  mac_addr_t src_addr_mac;
  int check_opcode = 0, check_ip = 0;

  timerms_t timer;
  timerms_reset(&timer, timeout);

  do {
    long int time_left = timerms_left(&timer);

    recv_message = eth_recv ( iface, src_addr_mac, ARP_ETH_TYPE, buffer, buf_len, time_left );

    if (recv_message < 0) {
      return -1;
    } else if (recv_message == 0) {
      /* Timeout! */
      return 1;
    } else if (recv_message < ARP_DATA_LENGTH) {
      fprintf(stderr, "eth_recv(): Trama de mensaje invalido: %d bytes\n",
              recv_message);
      continue;
    }

    arp_reply_ptr = (arp_message_t *)buffer;
    check_opcode = ntohs(arp_reply_ptr->opcode) == ARP_OPCODE_REPLY;
    check_ip = (memcmp(arp_reply_ptr->src_addr_ip, ip_addr, IPv4_ADDR_SIZE) == 0);
  } while(!(check_opcode && check_ip));

  memcpy(mac_addr, arp_reply_ptr->src_addr_mac, MAC_ADDR_SIZE);

  return 0;
}
