#include "../../files/ipv4_base/ipv4.h"
#include "../../files/eth_base/eth.h"
#include <stdio.h>
#include <libgen.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#define CONFIG_FILE "./files/ipv4_config/ipv4_config_server.txt"
#define ROUTE_TABLE_FILE "./files/ipv4_route_table/ipv4_route_table_server.txt"
#define MY_PROTO_IP 150

/* int main ( int argc, char * argv[] );
 *
 * DESCRIPCIÓN:
 *   Función principal del servidor IPv4. Lee los argumentos escritos
 *   en línea de comandos, espera datos del client y le responde con los mismos.
 *
 * PARÁMETROS:
 *   'argc': Número de parámetros pasados al programa por la línea de
 *           comandos, incluyendo el propio nombre del programa.
 *
 * VALOR DEVUELTO:
 *   Devuelve -1 si hay errores
 */
 int main ( int argc, char * argv[] ) {

  printf("Escuchando IPv4...\n");

  /* Abre fichero tabla de rutas*/
  ipv4_layer_t * ipv4_layer = ipv4_open(CONFIG_FILE, ROUTE_TABLE_FILE);
  int timeout = -1; /* Infintos milisegundos de timeout */

  unsigned char buffer_rcvd[IPv4_MTU];
  ipv4_addr_t ip_src;

  while(1) {
    int err_recv = ipv4_receive(ipv4_layer, ip_src, (uint8_t) MY_PROTO_IP, buffer_rcvd, IPv4_MTU, timeout);

    if(err_recv==-1){
      return -1;
    }

    if(err_recv==0){
      exit(0);
    }

    char ip_src_str[IPv4_STR_MAX_LENGTH];
    ipv4_addr_str(ip_src, ip_src_str);
    printf("%d bytes recibidos de: %s\n", err_recv, ip_src_str);
    print_pkt(buffer_rcvd,err_recv,0);

    int err_send = ipv4_send(ipv4_layer, ip_src, (uint8_t) MY_PROTO_IP, buffer_rcvd, err_recv);

    printf("%d bytes enviados.\n", err_send);

    if(err_send == -1){
      return -1;
    }
    if(err_send == 0){
      exit(0);
    }
  }

  int err_close = ipv4_close(ipv4_layer);

  if(err_close){
    return -1;
  }

 return 0;
}
